﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  /// <summary>
  /// An operator which crosses two double values. TODO
  /// </summary>
  [Item("MultiDoubleValueCrossover", "An operator which crosses two double values. It randomly selects and applies one of its crossovers every time it is called.")]
  [StorableClass]
  public class MultiDoubleValueCrossover : StochasticMultiBranch<IDoubleValueCrossover>, IDoubleValueCrossover, IStochasticOperator {
    public override bool CanChangeName {
      get { return false; }
    }
    protected override bool CreateChildOperation {
      get { return true; }
    }

    #region Constructors and Cloning
    public MultiDoubleValueCrossover()
      : base() {
      foreach (Type type in ApplicationManager.Manager.GetTypes(typeof(IDoubleValueCrossover)).OrderBy(op => op.Name)) {
        if (!typeof(MultiOperator<IDoubleValueCrossover>).IsAssignableFrom(type))
          Operators.Add((IDoubleValueCrossover)Activator.CreateInstance(type), true);
      }
    }

    [StorableConstructor]
    protected MultiDoubleValueCrossover(bool deserializing) : base(deserializing) { }
    protected MultiDoubleValueCrossover(MultiDoubleValueCrossover original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new MultiDoubleValueCrossover(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      if (Operators.Count == 0) throw new InvalidOperationException(Name + ": Please add at least one crossover to choose from.");
      return base.Apply();
    }

    public void Apply(IRandom random, DoubleValue value, DoubleValue other, DoubleValueRange range) {
      Operators[random.Next(Operators.Count)].Apply(random, value, other, range);
    }
  }
}
