﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.ParameterConfigurationEncoding {
  /// <summary>
  /// An operator which manipulates two double values. TODO
  /// </summary>
  [Item("NormalDoubleValueManipulator", "An operator which manipulates two double values. TODO")]
  [StorableClass]
  public class NormalDoubleValueManipulator : SingleSuccessorOperator, IDoubleValueManipulator, IStochasticOperator {
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }

    #region Constructors and Cloning
    public NormalDoubleValueManipulator() : base() { }

    [StorableConstructor]
    protected NormalDoubleValueManipulator(bool deserializing) : base(deserializing) { }
    protected NormalDoubleValueManipulator(NormalDoubleValueManipulator original, Cloner cloner) : base(original, cloner) { }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new NormalDoubleValueManipulator(this, cloner);
    }
    #endregion

    // TODO: override Apply
    public void Apply(IRandom random, DoubleValue value, DoubleValueRange range) {
      ApplyStatic(random, value, range);
    }

    public static void ApplyStatic(IRandom random, DoubleValue value, DoubleValueRange range) {
      var strategy = new RealVector(new double[] { (range.UpperBound.Value - range.LowerBound.Value) / 10 });
      var vector = new RealVector(1);
      double val = value.Value;

      do {
        vector[0] = val;
        NormalAllPositionsManipulator.Apply(random, vector, strategy);
        value.Value = vector[0];
        value.Value = range.ApplyStepSize(value.Value);
      } while (!range.IsInRange(value.Value));
    }
  }
}
